/*
 * MikroSDK - MikroE Software Development Kit
 * Copyright (c) 2019, MikroElektronika - www.mikroe.com
 * All rights reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

/*!
 * \file
 *
 */

#include "fan4.h"

// ------------------------------------------------ PUBLIC FUNCTION DEFINITIONS

void fan4_cfg_setup ( fan4_cfg_t *cfg )
{
    // Communication gpio pins 

    cfg->scl = HAL_PIN_NC;
    cfg->sda = HAL_PIN_NC;

    cfg->i2c_speed = I2C_STANDARD_SPEED; 
    cfg->i2c_address = FAN4_SLAVE_ADDRESS;
}

FAN4_RETVAL fan4_init ( fan4_t *ctx, fan4_cfg_t *cfg )
{
     i2c_master_config_t i2c_cfg;

    i2c_master_configure_default( &i2c_cfg );
    i2c_cfg.speed    = cfg->i2c_speed;
    i2c_cfg.pins.scl = cfg->scl;
    i2c_cfg.pins.sda = cfg->sda;

    ctx->slave_address = cfg->i2c_address;

    if (  i2c_master_open( &ctx->i2c, &i2c_cfg ) != I2C_SUCCESS )
    {
        return FAN4_INIT_ERROR;
    }

    return FAN4_OK;
}

void fan4_default_cfg ( fan4_t *ctx )
{
    // Click default configuration

    fan4_set_output( ctx, FAN4_MIN_VOLT_SCALE, FAN4_BOOST_START_TIMER_DIS );
}

void fan4_generic_write ( fan4_t *ctx, uint8_t cmd )
{
    i2c_master_transfer_t i2c_transfer;
    uint8_t tx_buf[ 5 ];

    tx_buf[ 0 ] = cmd;

    i2c_transfer.slave_address = ctx->slave_address;
    i2c_transfer.buffer   = tx_buf;
    i2c_transfer.count    = 1;
    i2c_transfer.end_mode = I2C_STOP_MODE;

    i2c_master_start( &ctx->i2c );
    i2c_master_write( &ctx->i2c, &i2c_transfer );
}

void fan4_generic_read ( fan4_t *ctx, uint8_t *data_out )
{
    i2c_master_transfer_t i2c_transfer;

    i2c_transfer.slave_address = ctx->slave_address;
    i2c_transfer.buffer   = data_out;
    i2c_transfer.count    = 1;
    i2c_transfer.end_mode = I2C_STOP_MODE;

    i2c_master_start( &ctx->i2c );
    i2c_master_read( &ctx->i2c, &i2c_transfer );
}

FAN4_RETVAL fan4_set_output ( fan4_t *ctx, uint16_t output_volt, uint8_t boost_start_timer )
{
    uint8_t cmdByte;
    float temp;

    if ( output_volt > FAN4_MAX_VOLT_SCALE )
    {
        return FAN4_VOLT_RANGE_CHECK_MASK;
    }
    if ( boost_start_timer > 1 )
    {
        return FAN4_BS_TIMER_CHECK_MASK;
    }

    temp = output_volt / FAN4_DAC_LSB;
    cmdByte = temp;
    cmdByte |= ( boost_start_timer << 6 );

    fan4_generic_write ( ctx, cmdByte );

    return FAN4_CHECK_OK;
}

uint8_t fan4_check_diagnostic ( fan4_t *ctx )
{
    uint8_t rx_data;

    fan4_generic_read( ctx, &rx_data );

    return rx_data;
}

// ------------------------------------------------------------------------- END

